@@ -50,7 +50,7 @@ import org.apache.catalina.util.CharsetMapper;
  * <p>
  *
  * @author Craig R. McClanahan
- * @version $Id: Context.java 953025 2010-06-09 14:30:30Z markt $
+ * @version $Id: Context.java 1027196 2010-10-25 17:00:43Z markt $
  */
 
 public interface Context extends Container {
@@ -325,6 +325,35 @@ public interface Context extends Container {
      * @param docBase The new document root
      */
     public void setDocBase(String docBase);
+    
+    
+    /**
+     * Is URL rewriting disabled?
+     * URL rewriting is an optional component of the servlet 2.5 specification.
+     * However if set to true this will be non-compliant with the specification
+     * as the specification requires that there <b>must</b> be a way to retain
+     * sessions if the client doesn't allow session cookies.
+     * 
+     * @return true If URL rewriting is disabled.
+     * 
+     * @see <a href="http://jcp.org/aboutJava/communityprocess/mrel/jsr154/index2.html">Servlet
+     *      2.5 Specification. Sections SRV.7.1.3 and SRV.7.1.4</a>
+     * @see javax.servlet.http.HttpServletResponse#encodeURL(String) encodeURL
+     * @see javax.servlet.http.HttpServletResponse#encodeRedirectURL(String)
+     *      encodeRedirectURL
+     */
+    public boolean isDisableURLRewriting();
+    
+    /**
+     * Is URL rewriting disabled?
+     * URL rewriting is an optional component of the servlet 2.5 specification.
+     * However if set to true this will be non-compliant with the specification
+     * as the specification requires that there <b>must</b> be a way to retain
+     * sessions if the client doesn't allow session cookies.
+     *
+     * @param disable True to disable URL Rewriting. Default <b>false</b>.
+     */
+    public void setDisableURLRewriting(boolean disable);
 
 
     /**
@@ -1095,10 +1124,6 @@ public interface Context extends Container {
      * @param xmlNamespaceAware true to enable namespace awareness
      */
     public void setXmlNamespaceAware(boolean xmlNamespaceAware);
-    /**
-     * Get the server.xml <context> attribute's xmlValidation.
-     * @return true if validation is enabled.
-     */
      
 
     /**
@@ -1110,9 +1135,9 @@ public interface Context extends Container {
 
 
     /**
-     * Get the server.xml <context> attribute's webXmlValidation.
+     * Get the validation feature of the XML parser used when
+     * parsing tlds files. 
      * @return true if validation is enabled.
-     *
      */
     public boolean getTldValidation();
 
@@ -1130,7 +1155,5 @@ public interface Context extends Container {
      * @param tldNamespaceAware true to enable namespace awareness
      */
     public void setTldNamespaceAware(boolean tldNamespaceAware);
-
-
 }
 
