@@ -71,7 +71,7 @@ import org.apache.juli.logging.LogFactory;
  * requests.  Requests of any other type will simply be passed through.
  *
  * @author Craig R. McClanahan
- * @version $Id: AuthenticatorBase.java 939336 2010-04-29 15:00:41Z kkolinko $
+ * @version $Id: AuthenticatorBase.java 993409 2010-09-07 16:05:25Z markt $
  */
 
 
@@ -797,7 +797,7 @@ public abstract class AuthenticatorBase
                 cookie.setDomain(ssoDomain);
             }
 
-            response.addCookie(cookie);
+            response.addCookieInternal(cookie, context.getUseHttpOnly());
 
             // Register this principal with our SSO valve
             sso.register(ssoId, principal, authType, username, password);
