@@ -51,7 +51,7 @@ import org.apache.tomcat.util.http.MimeHeaders;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Id: FormAuthenticator.java 939336 2010-04-29 15:00:41Z kkolinko $
+ * @version $Id: FormAuthenticator.java 1066314 2011-02-02 03:03:08Z kkolinko $
  */
 
 public class FormAuthenticator
@@ -246,8 +246,9 @@ public class FormAuthenticator
             return (false);
         }
 
-        // Yes -- Validate the specified credentials and redirect
-        // to the error page if they are not correct
+        // Yes -- Acknowledge the request, validate the specified credentials
+        // and redirect to the error page if they are not correct
+        response.sendAcknowledgement();
         Realm realm = context.getRealm();
         if (characterEncoding != null) {
             request.setCharacterEncoding(characterEncoding);
@@ -511,6 +512,9 @@ public class FormAuthenticator
         }
 
         if ("POST".equalsIgnoreCase(request.getMethod())) {
+            // May need to acknowledge a 100-continue expectation
+            request.getResponse().sendAcknowledgement();
+
             ByteChunk body = new ByteChunk();
             body.setLimit(request.getConnector().getMaxSavePostSize());
 
