@@ -70,6 +70,9 @@ import org.apache.catalina.util.Enumerator;
 import org.apache.catalina.util.ParameterMap;
 import org.apache.catalina.util.StringManager;
 import org.apache.catalina.util.StringParser;
+import org.apache.juli.logging.Log;
+import org.apache.juli.logging.LogFactory;
+import org.apache.tomcat.util.ExceptionUtils;
 
 
 /**
@@ -77,22 +80,28 @@ import org.apache.catalina.util.StringParser;
  *
  * @author Remy Maucherat
  * @author Craig R. McClanahan
- * @version $Id: Request.java 955665 2010-06-17 16:23:22Z kkolinko $
+ * @version $Id: Request.java 1066315 2011-02-02 03:09:54Z kkolinko $
  */
 
 public class Request
     implements HttpServletRequest {
 
+    private final static boolean ALLOW_EMPTY_QUERY_STRING;
 
-    // ----------------------------------------------------------- Constructors
-
-
+    private static final Log log = LogFactory.getLog(Request.class);
+    
     static {
         // Ensure that classes are loaded for SM
         new StringCache.ByteEntry();
         new StringCache.CharEntry();
+        
+        ALLOW_EMPTY_QUERY_STRING = Boolean.parseBoolean(System.getProperty(
+                "org.apache.catalina.connector.Request.ALLOW_EMPTY_QUERY_STRING",
+                Boolean.toString(Globals.STRICT_SERVLET_COMPLIANCE)));
     }
 
+    
+    // ----------------------------------------------------------- Constructors
     public Request() {
 
         formats[0].setTimeZone(GMT_ZONE);
@@ -172,6 +181,13 @@ public class Request
     protected HashMap attributes = new HashMap();
 
 
+    /**
+     * Flag that indicates if SSL attributes have been parsed to improve
+     * performance for applications (usually frameworks) that make multiple
+     * calls to {@link Request#getAttributeNames()}.
+     */
+    protected boolean sslAttributesParsed = false;
+
     /**
      * List of read only attributes for this Request.
      */
@@ -412,11 +428,17 @@ public class Request
         localName = null;
 
         attributes.clear();
+        sslAttributesParsed = false;
         notes.clear();
         cookies = null;
 
         if (session != null) {
-            session.endAccess();
+            try {
+                session.endAccess();
+            } catch (Throwable t) {
+                ExceptionUtils.handleThrowable(t);
+                log.warn(sm.getString("coyoteRequest.sessionEndAccessFail"), t);
+            }
         }
         session = null;
         requestedSessionCookie = false;
@@ -912,6 +934,7 @@ public class Request
                 attributes.put(Globals.SSL_SESSION_ID_ATTR, attr);
             }
             attr = attributes.get(name);
+            sslAttributesParsed = true;
         }
         return attr;
     }
@@ -955,7 +978,7 @@ public class Request
      * attributes and may also support additional attributes.
      */
     public Enumeration getAttributeNames() {
-        if (isSecure()) {
+        if (isSecure() && !sslAttributesParsed) {
             getAttribute(Globals.CERTIFICATES_ATTR);
         }
         return new Enumerator(attributes.keySet(), true);
@@ -1966,11 +1989,11 @@ public class Request
      */
     public String getQueryString() {
         String queryString = coyoteRequest.queryString().toString();
-        if (queryString == null || queryString.equals("")) {
-            return (null);
-        } else {
-            return queryString;
+        if (!ALLOW_EMPTY_QUERY_STRING && "".equals(queryString)) {
+            return null;
         }
+        
+        return queryString;
     }
 
 
@@ -2717,7 +2740,12 @@ public class Request
             int semi = entry.indexOf(";q=");
             if (semi >= 0) {
                 try {
-                    quality = Double.parseDouble(entry.substring(semi + 3));
+                    String strQuality = entry.substring(semi + 3);
+                    if (strQuality.length() <= 5) {
+                        quality = Double.parseDouble(strQuality);
+                    } else {
+                        quality = 0.0;
+                    }
                 } catch (NumberFormatException e) {
                     quality = 0.0;
                 }
