@@ -40,6 +40,7 @@ import org.apache.catalina.core.StandardContext;
 import org.apache.catalina.ha.CatalinaCluster;
 import org.apache.catalina.ha.ClusterMessage;
 import org.apache.catalina.ha.tcp.ReplicationValve;
+import org.apache.catalina.session.ManagerBase;
 import org.apache.catalina.tribes.Member;
 import org.apache.catalina.tribes.io.ReplicationStream;
 import org.apache.catalina.util.LifecycleSupport;
@@ -62,7 +63,7 @@ import org.apache.catalina.ha.ClusterManager;
  * @author Craig R. McClanahan
  * @author Jean-Francois Arcand
  * @author Peter Rossbach
- * @version $Id: DeltaManager.java 956838 2010-06-22 10:09:14Z kfujino $
+ * @version $Id: DeltaManager.java 1064254 2011-01-27 18:58:55Z kkolinko $
  */
 
 public class DeltaManager extends ClusterManagerBase{
@@ -645,6 +646,7 @@ public class DeltaManager extends ClusterManagerBase{
                         SessionMessage.EVT_CHANGE_SESSION_ID, data,
                         orgSessionID, orgSessionID + "-"
                                 + System.currentTimeMillis());
+                msg.setTimestamp(System.currentTimeMillis());
                 counterSend_EVT_CHANGE_SESSION_ID++;
                 send(msg);
             } catch (IOException e) {
@@ -1206,6 +1208,11 @@ public class DeltaManager extends ClusterManagerBase{
         DeltaSession session = null;
         try {
             session = (DeltaSession) findSession(sessionId);
+            if (session == null) {
+                // A parallel request has called session.invalidate() which has
+                // remove the session from the Manager.
+                return null;
+            }
             DeltaRequest deltaRequest = session.getDeltaRequest();
             session.lock();
             SessionMessage msg = null;
@@ -1245,7 +1252,8 @@ public class DeltaManager extends ClusterManagerBase{
             //check to see if we need to send out an access message
             if (!expires && (msg == null)) {
                 long replDelta = System.currentTimeMillis() - session.getLastTimeReplicated();
-                if (replDelta > (getMaxInactiveInterval() * 1000)) {
+                if (session.getMaxInactiveInterval() >=0 && 
+                        replDelta > (session.getMaxInactiveInterval() * 1000)) {
                     counterSend_EVT_SESSION_ACCESSED++;
                     msg = new SessionMessageImpl(getName(),
                                                  SessionMessage.EVT_SESSION_ACCESSED, 
@@ -1279,6 +1287,20 @@ public class DeltaManager extends ClusterManagerBase{
     public synchronized void resetStatistics() {
         processingTime = 0 ;
         expiredSessions = 0 ;
+        synchronized (sessionCreationTiming) {
+            sessionCreationTiming.clear();
+            while (sessionCreationTiming.size() <
+                    ManagerBase.TIMING_STATS_CACHE_SIZE) {
+                sessionCreationTiming.add(null);
+            }
+        }
+        synchronized (sessionExpirationTiming) {
+            sessionExpirationTiming.clear();
+            while (sessionExpirationTiming.size() <
+                    ManagerBase.TIMING_STATS_CACHE_SIZE) {
+                sessionExpirationTiming.add(null);
+            }
+        }
         rejectedSessions = 0 ;
         sessionReplaceCounter = 0 ;
         counterNoStateTransfered = 0 ;
@@ -1324,6 +1346,7 @@ public class DeltaManager extends ClusterManagerBase{
     protected void sessionExpired(String id) {
         counterSend_EVT_SESSION_EXPIRED++ ;
         SessionMessage msg = new SessionMessageImpl(getName(),SessionMessage.EVT_SESSION_EXPIRED, null, id, id+ "-EXPIRED-MSG");
+        msg.setTimestamp(System.currentTimeMillis());
         if (log.isDebugEnabled()) log.debug(sm.getString("deltaManager.createMessage.expire",getName(), id));
         send(msg);
     }
@@ -1618,13 +1641,15 @@ public class DeltaManager extends ClusterManagerBase{
     protected void handleCHANGE_SESSION_ID(SessionMessage msg,Member sender) throws IOException {
         counterReceive_EVT_CHANGE_SESSION_ID++;
         DeltaSession session = (DeltaSession) findSession(msg.getSessionID());
-        String newSessionID = deserializeSessionId(msg.getSession());
-        session.setPrimarySession(false);
-        if(notifySessionListenersOnReplication) {
-            session.setId(newSessionID);
-        } else {
-            session.setIdInternal(newSessionID);
-            add(session);
+        if (session != null) {
+            String newSessionID = deserializeSessionId(msg.getSession());
+            session.setPrimarySession(false);
+            if (notifySessionListenersOnReplication) {
+                session.setId(newSessionID);
+            } else {
+                session.setIdInternal(newSessionID);
+                add(session);
+            }
         }
     }
 
