@@ -68,7 +68,7 @@ import org.apache.tomcat.util.http.fileupload.FileItem;
 * @author Bip Thelin
 * @author Malcolm Edgar
 * @author Glenn L. Nielsen
-* @version $Id: HTMLManagerServlet.java 939518 2010-04-30 00:08:58Z kkolinko $
+* @version $Id: HTMLManagerServlet.java 1057270 2011-01-10 16:43:40Z jfclere $
 * @see ManagerServlet
 */
 
@@ -407,10 +407,11 @@ public final class HTMLManagerServlet extends ManagerServlet {
                 
                 args = new Object[7];
                 args[0] = URL_ENCODER.encode(displayPath);
-                args[1] = displayPath;
-                args[2] = context.getDisplayName();
-                if (args[2] == null) {
+                args[1] = RequestUtil.filter(displayPath);
+                if (context.getDisplayName() == null) {
                     args[2] = "&nbsp;";
+                } else {
+                    args[2] = RequestUtil.filter(context.getDisplayName());
                 }
                 args[3] = new Boolean(context.getAvailable());
                 args[4] = response.encodeURL
@@ -675,7 +676,7 @@ public final class HTMLManagerServlet extends ManagerServlet {
      * @see javax.servlet.Servlet#getServletInfo()
      */
     public String getServletInfo() {
-        return "HTMLManagerServlet, Copyright (c) 1999-2010, The Apache Software Foundation";
+        return "HTMLManagerServlet, Copyright (c) 1999-2011, The Apache Software Foundation";
     }   
     
     /**
@@ -1020,7 +1021,7 @@ public final class HTMLManagerServlet extends ManagerServlet {
         " <td class=\"row-left\" bgcolor=\"{6}\" rowspan=\"2\"><small>{2}</small></td>\n" +
         " <td class=\"row-center\" bgcolor=\"{6}\" rowspan=\"2\"><small>{3}</small></td>\n" +
         " <td class=\"row-center\" bgcolor=\"{6}\" rowspan=\"2\">" +
-        "<small><a href=\"{4}\" target=\"_blank\">{5}</a></small></td>\n";
+        "<small><a href=\"{4}\">{5}</a></small></td>\n";
 
     private static final String MANAGER_APP_ROW_BUTTON_SECTION =
         " <td class=\"row-left\" bgcolor=\"{13}\">\n" +
