@@ -59,6 +59,7 @@ import org.apache.catalina.util.StringManager;
 import org.apache.catalina.core.StandardContext;
 import org.apache.catalina.realm.GenericPrincipal;
 import org.apache.catalina.security.SecurityUtil;
+import org.apache.catalina.session.ManagerBase.SessionTiming;
 
 /**
  * Standard implementation of the <b>Session</b> interface.  This object is
@@ -78,7 +79,7 @@ import org.apache.catalina.security.SecurityUtil;
  * @author Craig R. McClanahan
  * @author Sean Legassick
  * @author <a href="mailto:jon@latchkey.com">Jon S. Stevens</a>
- * @version $Id: StandardSession.java 946841 2010-05-21 00:56:52Z kkolinko $
+ * @version $Id: StandardSession.java 1061433 2011-01-20 18:10:16Z markt $
  */
 
 public class StandardSession
@@ -762,6 +763,15 @@ public class StandardSession
                 manager.setSessionAverageAliveTime(average);
             }
 
+            if (manager instanceof ManagerBase) {
+                ManagerBase mb = (ManagerBase) manager;
+                SessionTiming timing = new SessionTiming(timeNow, timeAlive);
+                synchronized (mb.sessionExpirationTiming) {
+                    mb.sessionExpirationTiming.add(timing);
+                    mb.sessionExpirationTiming.poll();
+                }
+            }
+
             // Remove this session from our manager's active sessions
             manager.remove(this);
 
