@@ -33,6 +33,7 @@ import java.util.Set;
 
 import org.apache.catalina.tribes.Channel;
 import org.apache.catalina.tribes.ChannelException;
+import org.apache.catalina.tribes.ChannelException.FaultyMember;
 import org.apache.catalina.tribes.ChannelListener;
 import org.apache.catalina.tribes.Heartbeat;
 import org.apache.catalina.tribes.Member;
@@ -227,6 +228,10 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
             broadcast(MapMessage.MSG_START, true);
         } catch (ChannelException x) {
             log.warn("Unable to send map start message.");
+            // remove listener from channel
+            this.rpcChannel.breakdown();
+            this.channel.removeChannelListener(this);
+            this.channel.removeMembershipListener(this);
             throw new RuntimeException("Unable to start replicated map.",x);
         }
     }
@@ -249,6 +254,7 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
                                         channel.getLocalMember(false),
                                         null);
         if ( channel.getMembers().length > 0 ) {
+            try {
             //send a ping, wait for all nodes to reply
             Response[] resp = rpcChannel.send(channel.getMembers(), 
                                               msg, rpcChannel.ALL_REPLY, 
@@ -256,7 +262,14 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
                                               (int) accessTimeout);
             for (int i = 0; i < resp.length; i++) {
                 memberAlive(resp[i].getSource());
-            } //for
+                }
+            } catch (ChannelException ce) {
+                // Handle known failed members
+                FaultyMember[] faultyMembers = ce.getFaultyMembers();
+                for (FaultyMember faultyMember : faultyMembers) {
+                    memberDisappeared(faultyMember.getMember());
+                }
+            }            
         }
         //update our map of members, expire some if we didn't receive a ping back
         synchronized (mapMembers) {
@@ -866,12 +879,16 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
                     if ( dest!=null && dest.length >0) {
                         getChannel().send(dest, msg, getChannelSendOptions());
                     }
+                    if ( entry.getValue() != null && entry.getValue() instanceof ReplicatedMapEntry ) {
+                        ReplicatedMapEntry val = (ReplicatedMapEntry)entry.getValue();
+                        val.setOwner(getMapOwner());   
+                    }
                 }
                 entry.setPrimary(channel.getLocalMember(false));
                 entry.setBackupNodes(backup);
                 entry.setBackup(false);
                 entry.setProxy(false);
-
+                if ( getMapOwner()!=null ) getMapOwner().objectMadePrimay(key, entry.getValue());
 
             } catch (Exception x) {
                 log.error("Unable to replicate out data for a LazyReplicatedMap.get operation", x);
@@ -879,12 +896,6 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
             }
         }
         if (log.isTraceEnabled()) log.trace("Requesting id:"+key+" result:"+entry.getValue());
-        if ( entry.getValue() != null && entry.getValue() instanceof ReplicatedMapEntry ) {
-            ReplicatedMapEntry val = (ReplicatedMapEntry)entry.getValue();
-            //hack, somehow this is not being set above
-            val.setOwner(getMapOwner());
-            
-        }
         return entry.getValue();
     }    
 
