@@ -25,6 +25,7 @@ import java.nio.channels.Selector;
 import org.apache.coyote.ActionCode;
 import org.apache.coyote.OutputBuffer;
 import org.apache.coyote.Response;
+import org.apache.coyote.http11.filters.GzipOutputFilter;
 import org.apache.tomcat.util.buf.ByteChunk;
 import org.apache.tomcat.util.buf.CharChunk;
 import org.apache.tomcat.util.buf.MessageBytes;
@@ -34,7 +35,6 @@ import org.apache.tomcat.util.net.NioChannel;
 import org.apache.tomcat.util.net.NioEndpoint;
 import org.apache.tomcat.util.net.NioSelectorPool;
 import org.apache.tomcat.util.res.StringManager;
-import java.io.EOFException;
 import org.apache.tomcat.util.MutableInteger;
 
 /**
@@ -103,6 +103,12 @@ public class InternalNioOutputBuffer
     protected static StringManager sm =
         StringManager.getManager(Constants.Package);
 
+    /**
+     * Logger.
+     */
+    private static final org.apache.juli.logging.Log log
+        = org.apache.juli.logging.LogFactory.getLog(
+                InternalNioOutputBuffer.class);
 
     // ----------------------------------------------------- Instance Variables
 
@@ -296,6 +302,19 @@ public class InternalNioOutputBuffer
 
         }
 
+        // go through the filters and if there is gzip filter
+        // invoke it to flush
+        for (int i = 0; i <= lastActiveFilter; i++) {
+            if (activeFilters[i] instanceof GzipOutputFilter) {
+                if (log.isDebugEnabled()) {
+                    log.debug("Flushing the gzip filter at position " + i +
+                            " of the filter chain...");
+                }
+                ((GzipOutputFilter) activeFilters[i]).flush();
+                break;
+            }
+        }
+        
         // Flush the current buffer
         flushBuffer();
 
@@ -329,9 +348,10 @@ public class InternalNioOutputBuffer
 
         // Recycle Request object
         response.recycle();
-        socket.getBufHandler().getWriteBuffer().clear();
-
-        socket = null;
+        if (socket != null) {
+            socket.getBufHandler().getWriteBuffer().clear();
+            socket = null;
+        }
         pos = 0;
         lastActiveFilter = -1;
         committed = false;
