@@ -26,7 +26,7 @@ import org.apache.el.util.MessageFactory;
 /**
  * A helper class of Arithmetic defined by the EL Specification
  * @author Jacob Hookom [jacob@hookom.net]
- * @version $Id: ELArithmetic.java 939519 2010-04-30 00:12:23Z kkolinko $
+ * @version $Id: ELArithmetic.java 1056451 2011-01-07 18:43:39Z markt $
  */
 public abstract class ELArithmetic {
 
@@ -225,9 +225,12 @@ public abstract class ELArithmetic {
         final ELArithmetic delegate;
         if (BIGDECIMAL.matches(obj0, obj1))
             delegate = BIGDECIMAL;
-        else if (DOUBLE.matches(obj0, obj1))
-            delegate = DOUBLE;
-        else if (BIGINTEGER.matches(obj0, obj1))
+        else if (DOUBLE.matches(obj0, obj1)) {
+            if (BIGINTEGER.matches(obj0, obj1))
+                delegate = BIGDECIMAL;
+            else
+                delegate = DOUBLE;
+        } else if (BIGINTEGER.matches(obj0, obj1))
             delegate = BIGINTEGER;
         else
             delegate = LONG;
@@ -245,7 +248,7 @@ public abstract class ELArithmetic {
 
         final ELArithmetic delegate;
         if (BIGDECIMAL.matches(obj0, obj1))
-            delegate = BIGDECIMAL;
+            delegate = DOUBLE;
         else if (DOUBLE.matches(obj0, obj1))
             delegate = DOUBLE;
         else if (BIGINTEGER.matches(obj0, obj1))
@@ -267,9 +270,12 @@ public abstract class ELArithmetic {
         final ELArithmetic delegate;
         if (BIGDECIMAL.matches(obj0, obj1))
             delegate = BIGDECIMAL;
-        else if (DOUBLE.matches(obj0, obj1))
-            delegate = DOUBLE;
-        else if (BIGINTEGER.matches(obj0, obj1))
+        else if (DOUBLE.matches(obj0, obj1)) {
+            if (BIGINTEGER.matches(obj0, obj1))
+                delegate = BIGDECIMAL;
+            else
+                delegate = DOUBLE;
+        } else if (BIGINTEGER.matches(obj0, obj1))
             delegate = BIGINTEGER;
         else
             delegate = LONG;
@@ -307,9 +313,12 @@ public abstract class ELArithmetic {
         final ELArithmetic delegate;
         if (BIGDECIMAL.matches(obj0, obj1))
             delegate = BIGDECIMAL;
-        else if (DOUBLE.matches(obj0, obj1))
-            delegate = DOUBLE;
-        else if (BIGINTEGER.matches(obj0, obj1))
+        else if (DOUBLE.matches(obj0, obj1)) {
+            if (BIGINTEGER.matches(obj0, obj1))
+                delegate = BIGDECIMAL;
+            else
+                delegate = DOUBLE;
+        } else if (BIGINTEGER.matches(obj0, obj1))
             delegate = BIGINTEGER;
         else
             delegate = LONG;
